// osver.h

#ifndef OSVER_H
#define OSVER_H


#ifndef CreateFile
#error Error: Include <windows.h> before "osver.h"
#endif


// Windows 95 and later
// 4.00. 950      Windows 95
// 4.00.1111      Windows 95 OSR2   (950B)
// 4.00.1212      Windows 95 OSR2.1 (950B)
// 4.00.1214      Windows 95 OSR2.5 (950C)
// 4.10.1998      Windows 98
// 4.10.2222      Windows 98 Second Edition
// 4.90.3000      Windows Me

// Windows NT and later
// 3.10. 528      Windows NT 3.1
// 3.50. 807      Windows NT 3.5
// 3.51.1057      Windows NT 3.51
// 4.00.1381      Windows NT 4.0
// 5.00.2195      Windows 2000
// 5.01.2600      Windows XP
// 5.02.3790      Windows Server 2003
// 6.00           Windows Vista

// see "http://en.wikipedia.org/wiki/Windows_NT"


#ifdef _WIN64
#define NT_ONLY
#endif


#ifdef NT_ONLY
#define IsNT()       1
#else
#define IsNT()       (!( _osver & 0x8000 ))
#endif

#define IsWin95()    ( ! IsNT() && _winver == 0x400 )
#define IsWin98()    ( ! IsNT() && _winver == 0x40A )
#define IsWinME()    ( ! IsNT() && _winver == 0x45A )

#define IsWin95orLater()      ( ! IsNT() && _winver >= 0x400 )
#define IsWin98orLater()      ( ! IsNT() && _winver >= 0x40A )
#define IsWinMEorLater()      ( ! IsNT() && _winver >= 0x45A )

#define IsWinNT31()        ( IsNT() && _winver == 0x30A )
#define IsWinNT35()        ( IsNT() && _winver == 0x332 )
#define IsWinNT351()       ( IsNT() && _winver == 0x333 )
#define IsWinNT40()        ( IsNT() && _winver == 0x400 )
#define IsWin2000()        ( IsNT() && _winver == 0x500 )
#define IsWinXP()          ( IsNT() && _winver == 0x501 )
#define IsWinServer2003()  ( IsNT() && _winver == 0x502 )
#define IsWinVISTA()       ( IsNT() && _winver == 0x600 )

#define IsWinNT31orLater()       ( IsNT() && _winver >= 0x30A )
#define IsWinNT35orLater()       ( IsNT() && _winver >= 0x332 )
#define IsWinNT351orLater()      ( IsNT() && _winver >= 0x333 )
#define IsWinNT40orLater()       ( IsNT() && _winver >= 0x400 )
#define IsWin2000orLater()       ( IsNT() && _winver >= 0x500 )
#define IsWinXPorLater()         ( IsNT() && _winver >= 0x501 )
#define IsWinServer2003orLater() ( IsNT() && _winver >= 0x502 )
#define IsWinVISTAorLater()      ( IsNT() && _winver >= 0x600 )



#if ! defined _MSC_VER || defined NOCRT

#define _osver get_osver_()
#define _winver get_winver_()


static unsigned int get_osver_ ( void ) {

   static unsigned int osver ;

   if ( ! osver ) {

      OSVERSIONINFOA OsVersionInfo ;
      OsVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA) ;
      GetVersionExA ( & OsVersionInfo ) ;

      if ( OsVersionInfo.dwPlatformId != VER_PLATFORM_WIN32_NT ) osver = ( OsVersionInfo.dwBuildNumber & 0x7FFF ) | 0x8000 ;
      else                                                       osver = ( OsVersionInfo.dwBuildNumber & 0x7FFF ) ;
   }

   return osver ;
}


static unsigned int get_winver_ ( void ) {

   static unsigned int winver ;

   if ( ! winver ) {

      OSVERSIONINFOA OsVersionInfo ;
      OsVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA) ;
      GetVersionExA ( & OsVersionInfo ) ;

      winver = ( OsVersionInfo.dwMajorVersion << 8 ) + OsVersionInfo.dwMinorVersion ;
   }

   return winver ;
}


#endif // ! defined _MSC_VER || defined NOCRT



#endif // OSVER_H


