// stringex.h

#ifndef STRINGEX_H
#define STRINGEX_H


#ifdef __cplusplus
extern "C" {
#endif


size_t strlcpy ( char *dst, const char *src, size_t bufsiz ) ;
size_t wcslcpy ( wchar_t *dst, const wchar_t *src, size_t bufsiz ) ;
size_t mbslcpy ( char *dst, const char *src, size_t bufsiz ) ;

size_t strlcat ( char *dst, const char *src, size_t bufsiz ) ;
size_t wcslcat ( wchar_t *dst, const wchar_t *src, size_t bufsiz ) ;
size_t mbslcat ( char *dst, const char *src, size_t bufsiz ) ;

char *strmove ( char *dst, const char *src ) ;
wchar_t *wcsmove ( wchar_t *dst, const wchar_t *src ) ;

char *strinsert ( char *dst, const char *src ) ;
wchar_t *wcsinsert ( wchar_t *dst, const wchar_t *src ) ;

char *strlset ( char *dst, int c, size_t bufsiz ) ;
wchar_t *wcslset ( wchar_t *dst, int c, size_t bufsiz ) ;
char *mbslset ( char *dst, int c, size_t bufsiz ) ;

int stricmp ( const char *string1, const char *string2 ) ;
int wcsicmp ( const wchar_t *string1, const wchar_t *string2 ) ;
int mbsicmp ( const char *string1, const char *string2 ) ;
int strnicmp ( const char *string1, const char *string2, size_t count ) ;
int wcsnicmp ( const wchar_t *string1, const wchar_t *string2, size_t count ) ;
int mbsnicmp ( const char *string1, const char *string2, size_t count ) ;

int strrcmp ( const char *string1, const char *string2 ) ;
int wcsrcmp ( const wchar_t *string1, const wchar_t *string2 ) ;
int mbsrcmp ( const char *string1, const char *string2 ) ;
int strricmp ( const char *string1, const char *string2 ) ;
int wcsricmp ( const wchar_t *string1, const wchar_t *string2 ) ;
int mbsricmp ( const char *string1, const char *string2 ) ;

int strrncmp ( const char *string1, const char *string2, size_t count ) ;
int wcsrncmp ( const wchar_t *string1, const wchar_t *string2, size_t count ) ;
int mbsrncmp ( const char *string1, const char *string2, size_t count ) ;
int strrnicmp ( const char *string1, const char *string2, size_t count ) ;
int wcsrnicmp ( const wchar_t *string1, const wchar_t *string2, size_t count ) ;
int mbsrnicmp ( const char *string1, const char *string2, size_t count ) ;

int strheadcmp ( const char *string, const char *head ) ;
int wcsheadcmp ( const wchar_t *string, const wchar_t *head ) ;
int strheadicmp ( const char *string, const char *head ) ;
int wcsheadicmp ( const wchar_t *string, const wchar_t *head ) ;
int mbsheadicmp ( const char *string, const char *head ) ;

int strtailcmp ( const char *string, const char *tail ) ;
int wcstailcmp ( const wchar_t *string, const wchar_t *tail ) ;
int mbstailcmp ( const char *string, const char *tail ) ;
int strtailicmp ( const char *string, const char *tail ) ;
int wcstailicmp ( const wchar_t *string, const wchar_t *tail ) ;
int mbstailicmp ( const char *string, const char *tail ) ;

char *mbsstr ( const char *string1, const char *string2 ) ;
char *strstri ( const char *string1, const char *string2 ) ;
wchar_t *wcsstri ( const wchar_t *string1, const wchar_t *string2 ) ;
char *mbsstri ( const char *string1, const char *string2 ) ;

char *strrstr ( const char *string1, const char *string2 ) ;
wchar_t *wcsrstr ( const wchar_t *string1, const wchar_t *string2 ) ;
char *mbsrstr ( const char *string1, const char *string2 ) ;
char *strrstri ( const char *string1, const char *string2 ) ;
wchar_t *wcsrstri ( const wchar_t *string1, const wchar_t *string2 ) ;
char *mbsrstri ( const char *string1, const char *string2 ) ;

char *mbschr ( const char *string, int c ) ;
char *strchri ( const char *string, int c ) ;
wchar_t *wcschri ( const wchar_t *string, int c ) ;
char *mbschri ( const char *string, int c ) ;

char *mbsrchr ( const char *string, int c ) ;
char *strrchri ( const char *string, int c ) ;
wchar_t *wcsrchri ( const wchar_t *string, int c ) ;
char *mbsrchri ( const char *string, int c ) ;

size_t mbsspn ( const char *string1, const char *string2 ) ;
size_t mbscspn ( const char *string1, const char *string2 ) ;

char *strspnp ( const char *string1, const char *string2 ) ;
wchar_t *wcsspnp ( const wchar_t *string1, const wchar_t *string2 ) ;
char *mbsspnp ( const char *string1, const char *string2 ) ;
char *mbspbrk ( const char *string1, const char *string2 ) ;

char *strlwr ( char *string ) ;
wchar_t *wcslwr ( wchar_t *string ) ;
char *mbslwr ( char *string ) ;
char *strupr ( char *string ) ;
wchar_t *wcsupr ( wchar_t *string ) ;
char *mbsupr ( char *string ) ;

char *mbsrev ( char *string ) ;

char *strtrunc ( char *string, size_t len ) ;
wchar_t *wcstrunc ( wchar_t *string, size_t len ) ;
char *mbstrunc ( char *string, size_t len ) ;

size_t mbslen ( const char *string ) ;

int mbsbtype ( const char *string, size_t count ) ;
int ismbslead ( const char *string, const char *current ) ;
int ismbstrail ( const char *string, const char *current ) ;

char *strtokex ( char *string, char **next ) ;
wchar_t *wcstokex ( wchar_t *string, wchar_t **next ) ;

wchar_t *wmemchr ( const wchar_t *string, int c, size_t count ) ;
char *mbsmemchr ( const char *string, int c, size_t count ) ;

void *memrchr ( const void *string, int c, size_t count ) ;
wchar_t *wmemrchr ( const wchar_t *string, int c, size_t count ) ;
char *mbsmemrchr ( const char *string, int c, size_t count ) ;

int wmemcmp ( const wchar_t *string1, const wchar_t *string2, size_t count ) ;

int memicmp ( const void *string1, const void *string2, size_t count ) ;
int wmemicmp ( const wchar_t *string1, const wchar_t *string2, size_t count ) ;
int mbsmemicmp ( const char *string1, const char *string2, size_t count ) ;

wchar_t *wmemset ( wchar_t *dst, int c, size_t bufsiz ) ;


#ifdef __cplusplus
}
#endif




#ifdef DEBUG_CONST_STRING     // for debug
#ifdef __cplusplus


#define FUNC_STR_STR(func,type)  \
   static inline const type *func##_ ( const type *s1, const type *s2 ) { return func ( s1, s2 ) ; } \
   static inline       type *func##_ (       type *s1, const type *s2 ) { return func ( s1, s2 ) ; }

#define FUNC_STR_INT(func,type)  \
   static inline const type *func##_ ( const type *s, int c ) { return func ( s, c ) ; } \
   static inline       type *func##_ (       type *s, int c ) { return func ( s, c ) ; }

#define FUNC_STR_INT_SIZET(func,type)  \
   static inline const type *func##_ ( const type *s, int c, size_t i ) { return func ( s, c, i ) ; } \
   static inline       type *func##_ (       type *s, int c, size_t i ) { return func ( s, c, i ) ; }


FUNC_STR_STR ( mbsstr, char )
FUNC_STR_STR ( strstri, char )
FUNC_STR_STR ( wcsstri, wchar_t )
FUNC_STR_STR ( mbsstri, char )

FUNC_STR_STR ( strrstr, char )
FUNC_STR_STR ( wcsrstr, wchar_t )
FUNC_STR_STR ( mbsrstr, char )
FUNC_STR_STR ( strrstri, char )
FUNC_STR_STR ( wcsrstri, wchar_t )
FUNC_STR_STR ( mbsrstri, char )

FUNC_STR_INT ( mbschr, char )
FUNC_STR_INT ( strchri, char )
FUNC_STR_INT ( wcschri, wchar_t )
FUNC_STR_INT ( mbschri, char )

FUNC_STR_INT ( mbsrchr, char )
FUNC_STR_INT ( strrchri, char )
FUNC_STR_INT ( wcsrchri, wchar_t )
FUNC_STR_INT ( mbsrchri, char )

FUNC_STR_STR ( strspnp, char )
FUNC_STR_STR ( wcsspnp, wchar_t )
FUNC_STR_STR ( mbsspnp, char )
FUNC_STR_STR ( mbspbrk, char )

FUNC_STR_INT_SIZET ( wmemchr, wchar_t )
FUNC_STR_INT_SIZET ( mbsmemchr, char )

FUNC_STR_INT_SIZET ( memrchr, void )
FUNC_STR_INT_SIZET ( wmemrchr, wchar_t )
FUNC_STR_INT_SIZET ( mbsmemrchr, char )


#undef FUNC_STR_STR
#undef FUNC_STR_INT
#undef FUNC_STR_INT_SIZET


#define mbsstr mbsstr_
#define strstri strstri_
#define wcsstri wcsstri_
#define mbsstri mbsstri_

#define strrstr strrstr_
#define wcsrstr wcsrstr_
#define mbsrstr mbsrstr_
#define strrstri strrstri_
#define wcsrstri wcsrstri_
#define mbsrstri mbsrstri_

#define mbschr mbschr_
#define strchri strchri_
#define wcschri wcschri_
#define mbschri mbschri_

#define mbsrchr mbsrchr_
#define strrchri strrchri_
#define wcsrchri wcsrchri_
#define mbsrchri mbsrchri_

#define strspnp strspnp_
#define wcsspnp wcsspnp_
#define mbsspnp mbsspnp_
#define mbspbrk mbspbrk_

#define memrchr memrchr_
#define wmemrchr wmemrchr_


#endif // __cplusplus
#endif // DEBUG_CONST_STRING



#endif


