// winmain.cpp

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <mbctype.h>
#include "msc.h"
#include "winmain.h"



// memo -D
//   NOCRT     Do not use C-Runtime Library
//   NOARG     Declared main without argc, argv
//   NT_ONLY   Cannot run on 95/98/Me platforms.




// select UNICODE entry point compulsorily
#if defined _MSC_VER && ! defined _UNICODE
#define _UNICODE
#endif



#if defined DEBUG_CONST_STRING && defined NOCRT
#undef NOCRT
#define NOARG
#endif



#ifdef __cplusplus
extern "C" {
#endif



#ifdef _UNICODE
static int SetArgumentA ( void ) ;
#else
static int SetArgumentW ( void ) ;
#endif



static int DisplayCriticalError ( void ) ;
static int DisplayPlatformError ( void ) ;
static void __cdecl AlertOnClose ( void ) ;



#if defined _CONSOLE || defined _WINDOWS && ! defined _USRDLL
static const char szCriticalError [] = "Critical error - not enough space for arguments.\n" ;
#endif

#if defined _CONSOLE || defined _WINDOWS && ! defined _USRDLL
#if defined NT_ONLY && ! defined NOCRT
static const char szPlatformError [] = "Critical error - cannot be run on 95/98/Me.\n" ;
#endif
#endif



#if defined _CONSOLE
static const char szAlertOnClose [] = "Press any key to close..." ;
#endif



#if defined _WINDOWS && ! defined _USRDLL
#define GETPROCADDRESS(dll,type,function) ( ! dll || ! function && ( function = (type) GetProcAddress ( dll, #function ) ) == NULL )
#define MessageBoxA MessageBoxA_
typedef int ( WINAPI *MESSAGEBOXA ) ( HWND hWnd, LPCSTR lpText, LPCSTR lpCaption, UINT uType ) ;
static HINSTANCE hUser32 ;
static MESSAGEBOXA MessageBoxA ;
#endif



////////////////////////////////////////////
//                  COMMON                //
////////////////////////////////////////////



#ifdef _MSC_VER
int __cdecl _setenvp ( void ) { return 0 ; }
int __cdecl _wsetenvp ( void ) { return 0 ; }
char *__cdecl __crtGetEnvironmentStringsA ( void ) { return NULL ; }
wchar_t *__cdecl __crtGetEnvironmentStringsW ( void ) { return NULL ; }
#endif // _MSC_VER



#if defined _WINDOWS && defined _USRDLL || defined NOARG
#ifdef _MSC_VER
int __cdecl _setargv ( void ) { return 0 ; }
int __cdecl _wsetargv ( void ) { return 0 ; }
char *__cdecl __crtGetCommandLineA ( void ) { return NULL ; }
wchar_t *__cdecl __crtGetCommandLineW ( void ) { return NULL ; }
#endif // _MSC_VER
#endif // defined _WINDOWS && defined _USRDLL



#if defined NOCRT
#ifdef _MSC_VER
#pragma comment ( linker, "/nodefaultlib" )
#ifdef _USRDLL
#pragma comment ( linker, "/entry:DllMain" )
#else
#pragma comment ( linker, "/entry:main" )
#undef main
int main ( void ) { return winmain_ () ; }
#endif
#endif // _MSC_VER
#endif // defined NOCRT



////////////////////////////////////////////
//                 WINDOWS                //
////////////////////////////////////////////



#if defined _WINDOWS && ! defined _USRDLL && ! defined NOCRT



#ifdef _UNICODE
int WINAPI wWinMain ( HINSTANCE hInstance,
                      HINSTANCE hPrevInstance,
                      LPSTR     lpCmdLine,
                      int       nCmdShow )
{

#if defined NT_ONLY && ! defined NOCRT
   if ( _osver & 0x8000 ) return DisplayPlatformError () ;
#endif

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentA () ) return DisplayCriticalError () ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#else
int WINAPI WinMain ( HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow )
{

#if defined NT_ONLY && ! defined NOCRT
   if ( _osver & 0x8000 ) return DisplayPlatformError () ;
#endif

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentW () ) return DisplayCriticalError () ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#endif



static int DisplayCriticalError ( void ) {
   hUser32 = LoadLibraryA ( "user32.dll" ) ;
   if ( GETPROCADDRESS ( hUser32, MESSAGEBOXA, MessageBoxA ) ) return 255 ;
   MessageBoxA ( NULL, szCriticalError, "", MB_OK | MB_ICONHAND | MB_SETFOREGROUND ) ;
   return 255 ;
}



#if defined NT_ONLY && ! defined NOCRT
static int DisplayPlatformError ( void ) {
   hUser32 = LoadLibraryA ( "user32.dll" ) ;
   if ( GETPROCADDRESS ( hUser32, MESSAGEBOXA, MessageBoxA ) ) return 255 ;
   MessageBoxA ( NULL, szPlatformError, "", MB_OK | MB_ICONHAND | MB_SETFOREGROUND ) ;
   return 255 ;
}
#endif



#ifdef _MSC_VER
char *__cdecl _wincmdln ( void ) { return NULL ; }
wchar_t *__cdecl _wwincmdln ( void ) { return NULL ; }
#endif // _MSC_VER


#endif // defined _WINDOWS && ! defined _USRDLL && ! defined NOCRT



////////////////////////////////////////////
//                 CONSOLE                //
////////////////////////////////////////////



#if defined _CONSOLE && ! defined NOCRT



#ifdef _UNICODE
#undef main
int wmain ( int argc, wchar_t *argv [] ) {

   atexit ( AlertOnClose ) ;

#if defined NT_ONLY && ! defined NOCRT
   if ( _osver & 0x8000 ) return DisplayPlatformError () ;
#endif

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentA () ) return DisplayCriticalError () ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#else
#undef main
int main ( int argc, char *argv [] ) {

   atexit ( AlertOnClose ) ;

#if defined NT_ONLY && ! defined NOCRT
   if ( _osver & 0x8000 ) return DisplayPlatformError () ;
#endif

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentW () ) return DisplayCriticalError () ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#endif



static int DisplayCriticalError ( void ) {
   fputs ( szCriticalError, stderr ) ;
   return 255 ;
}



#if defined NT_ONLY && ! defined NOCRT
static int DisplayPlatformError ( void ) {
   fputs ( szPlatformError, stderr ) ;
   return 255 ;
}
#endif



#endif // defined _CONSOLE && ! defined NOCRT



////////////////////////////////////////////
//                 ALERT                  //
////////////////////////////////////////////



#if defined _CONSOLE && ! defined NOCRT


static int IsEnabled = 0 ;
static int IsDisabled = 0 ;


static void WaitKey ( void ) ;



void EnableAlertOnClose ( void ) {
   IsEnabled = 1 ;
   return ;
}



void DisableAlertOnClose ( void ) {
   IsDisabled = 1 ;
   return ;
}



// GNXv[璼ڌĂяoꂽAOɃL[͂𑣂
static void __cdecl AlertOnClose ( void ) {

   if ( ( IsEnabled || ! IsDisabled && __argc <= 1 && ! IsConsole () ) && fisatty ( stdout ) && fisatty ( stderr ) ) {
      fflush ( NULL ) ;
      fputs ( szAlertOnClose, stderr ) ;
      fflush ( stderr ) ;
      WaitKey () ;
   }

   return ;
}



// R\[L[͂琧Ԃ
// W͂_CNgĂÂ܂ܐԂ
static void WaitKey ( void ) {

   HANDLE hConsole = fget_osfhandle ( stdin ) ;
   if ( hConsole == INVALID_HANDLE_VALUE ) return ;

   unsigned long dwDummy ;
   if ( ! GetConsoleMode ( hConsole, & dwDummy ) ) return ;

   while ( 1 ) {
      INPUT_RECORD InputRecord ;
      unsigned long nReadLength ;
      if ( ! ReadConsoleInput ( hConsole, & InputRecord, 1, & nReadLength ) || ! nReadLength ) break ;
      if ( InputRecord.EventType == KEY_EVENT && InputRecord.Event.KeyEvent.bKeyDown ) break ;
   }

   return ;
}



// R\[NꂽȂ 1 Ԃ
int IsConsole ( void ) {

   STARTUPINFO StartupInfo = { 0 } ;
   StartupInfo.cb = sizeof(STARTUPINFO) ;
   GetStartupInfo ( & StartupInfo ) ;

   if ( StartupInfo.dwFlags & STARTF_USESHOWWINDOW ) return 0 ;
   else                                              return 1 ;

}



#endif // defined _CONSOLE && ! defined NOCRT



////////////////////////////////////////////
//              GETARGUMENT               //
////////////////////////////////////////////



#if ! defined NOARG && ( defined _CONSOLE || defined _WINDOWS && ! defined _USRDLL ) && ! defined NOCRT


#ifndef _MSC_VER
wchar_t **__wargv ;
#endif



// 擾 (ANSI)
char *GetArgumentA ( int nCount ) {
   return __argv [ nCount ] ;
}



// 擾 (UNICODE)
wchar_t *GetArgumentW ( int nCount ) {
   return __wargv [ nCount ] ;
}



#ifdef _UNICODE
//  (ANSI)
//  0 As 1 Ԃ
static int SetArgumentA ( void ) {

   if ( ! ( __argv = (char**) malloc ( ( __argc + 1 ) * sizeof(char*) ) ) ) return 1 ;

   for ( int nCount = 0 ; nCount <= __argc ; nCount ++ ) {
      if ( ! __wargv [ nCount ] ) __argv [ nCount ] = NULL ;
      else {
         int nLength ;
         if ( ! ( nLength = WideCharToMultiByte ( CP_ACP, 0, __wargv [ nCount ], -1, NULL, 0, NULL, NULL ) ) ) return 1 ;
         if ( ! ( __argv [ nCount ] = (char*) malloc ( nLength * sizeof(char) ) ) ) return 1 ;
         if ( ! ( nLength = WideCharToMultiByte ( CP_ACP, 0, __wargv [ nCount ], -1, __argv [ nCount ], nLength, NULL, NULL ) ) ) return 1 ;
      }
   }

   return 0 ;
}
#else
//  (UNICODE)
//  0 As 1 Ԃ
static int SetArgumentW ( void ) {

   if ( ! ( __wargv = (wchar_t**) malloc ( ( __argc + 1 ) * sizeof(wchar_t*) ) ) ) return 1 ;

   for ( int nCount = 0 ; nCount <= __argc ; nCount ++ ) {
      if ( ! __argv [ nCount ] ) __wargv [ nCount ] = NULL ;
      else {
         int nLength ;
         if ( ! ( nLength = MultiByteToWideChar ( CP_ACP, 0, __argv [ nCount ], -1, NULL, 0 ) ) ) return 1 ;
         if ( ! ( __wargv [ nCount ] = (wchar_t*) malloc ( nLength * sizeof(wchar_t) ) ) ) return 1 ;
         if ( ! ( nLength = MultiByteToWideChar ( CP_ACP, 0, __argv [ nCount ], -1, __wargv [ nCount ], nLength ) ) ) return 1 ;
      }
   }

   return 0 ;
}
#endif



#endif // ! defined NOARG && ( defined _CONSOLE || defined _WINDOWS && ! defined _USRDLL ) && ! defined NOCRT



////////////////////////////////////////////
//            GETWINMAINCMDLINE           //
////////////////////////////////////////////



#if defined NOARG && ! defined _USRDLL


#ifdef NOCRT
#undef ismbblead
#define ismbblead IsDBCSLeadByte
#endif



// WinMain `̈擾 (ANSI)
char *GetWinMainCmdLineA ( void ) {

   char *szCommandLine ;
   static char szDummy [] = "" ;
   int IsInQuote = 0 ;

   szCommandLine = GetCommandLineA () ;
   if ( ! szCommandLine ) szCommandLine = szDummy ;

   while ( *szCommandLine > 0x20 || *szCommandLine && IsInQuote ) {
      if ( *szCommandLine == '\"' ) IsInQuote = ! IsInQuote ;
      if ( ismbblead ( (unsigned char) *szCommandLine ) && *( szCommandLine + 1 ) ) szCommandLine ++ ;
      szCommandLine ++ ;
   }

   while ( *szCommandLine && *szCommandLine <= 0x20 ) szCommandLine ++ ;

   return szCommandLine ;
}


// WinMain `̈擾 (UNICODE)
wchar_t *GetWinMainCmdLineW ( void ) {

   wchar_t *szCommandLine ;
   static wchar_t szDummy [] = L"" ;
   int IsInQuote = 0 ;

   szCommandLine = GetCommandLineW () ;
   if ( ! szCommandLine ) szCommandLine = szDummy ;

   while ( *szCommandLine > 0x20 || *szCommandLine && IsInQuote ) {
      if ( *szCommandLine == '\"' ) IsInQuote = ! IsInQuote ;
      szCommandLine ++ ;
   }

   while ( *szCommandLine && *szCommandLine <= 0x20 ) szCommandLine ++ ;

   return szCommandLine ;
}



#endif // defined NOARG && ! defined _USRDLL



#ifdef __cplusplus
}
#endif



